using System;
using System.Collections.Generic;
using System.Drawing.Design;
using System.Text;
using System.Xml;

namespace IP.Components
{
    partial class HostToolbox
    {
        private const string Serializer_TypeName = "type";

        /// <summary>
        /// Serializes a <paramref name="tab"/> properties into the <paramref name="category"/> element.
        /// </summary>
        /// <param name="category">A <see cref="XmlElement"/> representing the serialized <see cref="Toolbox.Tab"/> object.</param>
        /// <param name="tab">A <see cref="Toolbox.Tab"/> to serialize.</param>
        protected override void SerializeCategoryProperties(XmlElement category, Toolbox.Tab tab)
        {
            category.SetAttribute(Serializer_Opened, tab.Opened.ToString());
            if (tab == GeneralCategory)
                category.SetAttribute(Serializer_General, bool.TrueString);
        }

        /// <summary>
        /// Deserializes a <paramref name="category"/> XML element to <paramref name="tab"/> value.
        /// </summary>
        /// <param name="category">A <see cref="XmlElement"/> representation of serialized <see cref="Toolbox.Tab"/> object.</param>
        /// <param name="tab">A <see cref="Toolbox.Tab"/> to deserialize.</param>
        protected override void DeserializeCategoryProperties(XmlElement category, Toolbox.Tab tab)
        {
            bool boolValue;

            if (GetAttribute(category, Serializer_Opened, out boolValue))
                tab.Opened = boolValue;
        }
    }
}
