using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing.Design;
using System.Text;

namespace IP.Components
{
    partial class HostToolbox
    {
        private class ToolboxItemList : IList
        {
            // Used only Add, Remove, Count, GetEnumerator

            private Tab _tab;
            private HostToolbox _toolbox;
            
            public ToolboxItemList(Tab tab)
            {
                _tab = tab;
                _toolbox = _tab.Toolbox as HostToolbox;
            }

            public ToolboxItemList(HostToolbox toolbox)
            {
                _toolbox = toolbox;
            }

            #region IList Members

            public int Add(object value)
            {
                ToolboxItemContainer item = (ToolboxItemContainer)value;
                if (item != null)
                {
                    HostItem hostItem = new HostItem(item.GetToolboxItem(null));
                    if (_tab != null)
                        _tab.Items.Add(hostItem);
                    else if (_toolbox.GeneralCategory != null)
                        _toolbox.GeneralCategory.Items.Add(hostItem);
                    else
                    {
                        if (_toolbox.Categories.Count == 0)
                            _toolbox.Categories.Add(_toolbox.CreateNewTab(Properties.Resources.ToolboxTabDefaultName));
                        _toolbox.Categories[_toolbox.Categories.Count - 1].Items.Add(hostItem);
                    }
                }
                return -1;
            }

            public void Clear()
            {
            }

            public bool Contains(object value)
            {
                return false;
            }

            public int IndexOf(object value)
            {
                return -1;
            }

            public void Insert(int index, object value)
            {
            }

            public bool IsFixedSize
            {
                get { return false; }
            }

            public void Remove(object value)
            {
                ToolboxItemContainer item = (ToolboxItemContainer)value;
                if (item != null)
                {
                    ToolboxItem toolboxItem = item.GetToolboxItem(null);
                    _toolbox.RemoveToolboxItem(toolboxItem, (_tab != null) ? _tab.Text : null);
                }
            }

            public void RemoveAt(int index)
            {
            }

            object IList.this[int index]
            {
                get
                {
                    return null;
                }
                set
                {
                }
            }

            public bool IsReadOnly
            {
                get { return false; }
            }

            #endregion

            #region ICollection Members

            public void CopyTo(Array array, int index)
            {
            }

            public int Count
            {
                get 
                {
                    if (_tab != null)
                        return _tab.Items.Count;
                    else
                    {
                        int count = 0;
                        foreach (Tab tab in _toolbox.Categories)
                        {
                            count += tab.Items.Count;
                        }
                        return count;
                    }
                }
            }

            public bool IsSynchronized
            {
                get { return false; }
            }

            public object SyncRoot
            {
                get { return this; }
            }

            #endregion

            #region IEnumerable Members

            public IEnumerator GetEnumerator()
            {
                if (_tab != null)
                {
                    foreach (Item item in _tab.Items)
                    {
                        HostItem hostItem = item as HostItem;
                        if (hostItem != null)
                        {
                            yield return new ToolboxItemContainer(hostItem.ToolboxItem);
                        }
                    }
                }
                else
                {
                    foreach (Tab tab in _toolbox.Categories)
                    {
                        foreach (Item item in tab.Items)
                        {
                            HostItem hostItem = item as HostItem;
                            if (hostItem != null)
                            {
                                yield return new ToolboxItemContainer(hostItem.ToolboxItem);
                            }
                        }
                    }
                }
            }

            #endregion
        }
    }
}
