using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.ComponentModel.Design.Serialization;
using System.Data;
using System.Drawing;
using System.Drawing.Design;
using System.Text;
using System.Windows.Forms;

using IP.Components;

namespace TestToolbox
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            _surface = new DesignSurface();
            IServiceContainer serviceContainer = _surface.GetService(typeof(IServiceContainer)) as IServiceContainer;
            serviceContainer.AddService(typeof(IToolboxService), toolbox1.Service);
            serviceContainer.AddService(typeof(INameCreationService), new NameCreationService());
            _host = _surface.GetService(typeof(IDesignerHost)) as IDesignerHost;
            Control form = (Control) _host.CreateComponent(typeof(Form));
            form.Text = "Demo Form";

            //
            // View
            //
            _view = _surface.View as Control;
            _view.Dock = DockStyle.Fill;
            _view.Parent = splitContainer1.Panel2;

            this.PopulateToolBox();

            this.toolbox1.DoubleClick += new EventHandler(OnToolboxDoubleClick);
        }

        private void PopulateToolBox()
        {
            ToolboxItem item = new ToolboxItem();
            
            this.toolbox1.AddToolboxItem(new ToolboxItem(typeof(Label)), "Windows Controls");
            this.toolbox1.AddToolboxItem(new ToolboxItem(typeof(TextBox)), "Windows Controls");
            this.toolbox1.AddToolboxItem(new ToolboxItem(typeof(CheckBox)), "Windows Controls");
            this.toolbox1.AddToolboxItem(new ToolboxItem(typeof(Button)), "Windows Buttons");
            this.toolbox1.AddToolboxItem(new ToolboxItem(typeof(RadioButton)), "Windows Buttons");
            this.toolbox1.AddToolboxItem(new ToolboxItem(typeof(ImageList)), "Components");
            this.toolbox1.AddToolboxItem(new ToolboxItem(typeof(Timer)), "Components");
            this.toolbox1.AddToolboxItem(new ToolboxItem(typeof(Toolbox)), "Toolbox");

            Bitmap bitmap = new Bitmap(Properties.Resources.TimeStart);
            Toolbox.Item myItem = new Toolbox.Item("Time Starter", bitmap, bitmap.GetPixel(0, 0));
            this.toolbox1.Categories["Toolbox"].Items.Add(myItem);
        }

        private void disableAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem menuItem = (ToolStripMenuItem)sender;
            if (menuItem.Checked)
                toolbox1.EnableItems();
            else
                toolbox1.DisableItems();
            menuItem.Checked = !menuItem.Checked;
        }

        private void hideTabsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem menuItem = (ToolStripMenuItem)sender;
            if (menuItem.Checked)
                toolbox1.ShowTabs();
            else
                toolbox1.HideTabs();
            menuItem.Checked = !menuItem.Checked;
        }

        private void OnItemSelected(object sender, ItemSelectionEventArgs e)
        {
            if (toolbox1.SelectedItem != null)
                _itemLabel.Text = toolbox1.SelectedItem.Text;
            else
                _itemLabel.Text = "[None]";
        }

        private void OnToolboxDoubleClick(object sender, EventArgs e)
        {
            Toolbox.Item item = this.toolbox1.SelectedItem;
            if (item != null && item.Text == "Time Starter")
            {
                MessageBox.Show("It's OK!");
            }
        }

        private void OnSaveToolbox(object sender, EventArgs e)
        {
            this.toolbox1.Save("toolbox.xml");
        }

        private void OnLoadToolbox(object sender, EventArgs e)
        {
            this.toolbox1.Load("toolbox.xml");
        }

        class DemoToolbox : HostToolbox
        {
            public DemoToolbox()
                : base(true)
            {
            }

            public void AddToolboxItem(Type itemType, string tabName)
            {
                AddToolboxItem(new ToolboxItem(itemType), tabName);
            }

            public void AddToolboxItem(ToolboxItem item, string tabName)
            {
                Toolbox.Tab tab = this.Categories[tabName];
                if (tab == null)
                {
                    tab = new Tab(tabName);
                    this.Categories.Add(tab);
                }

                tab.Items.Add(new HostToolbox.HostItem(item));
            }

            public void DisableItems()
            {
                ChangeItemsState(false);
            }

            public void EnableItems()
            {
                ChangeItemsState(true);
            }

            public void HideTabs()
            {
                ChangeTabsState(false);
            }

            public void ShowTabs()
            {
                ChangeTabsState(true);
            }

            private void ChangeItemsState(bool enable)
            {
                foreach (Tab tab in Categories)
                {
                    foreach (Item item in tab.Items)
                        item.Enabled = enable;
                    tab.PointerItem.Enabled = enable;
                }
                Invalidate();
            }

            private void ChangeTabsState(bool show)
            {
                foreach (Tab tab in Categories)
                {
                    if (tab != GeneralCategory)
                    {
                        tab.Visible = show;
                    }
                }
            }
        }
    }
}